#!/bin/bash
# cpanel Fix Account Permissions
# https://github.com/PeachFlame/cPanel-fixperms
# https://www.thecpaneladmin.com/fix-account-permissions/
# 帳號備份檔，請放置 /mnt/account 目錄，最好是放在SSD硬碟
# 執行此 Script 之前，請先執行 find_sort_accounts.sh 產生帳號列表

date >> /mnt/restore_time2.txt

filename="/mnt/big_account_list2"

mkdir -p /mnt/account2_tmp

exec < $filename
while read line
do

if [ -e /home/$line ]; then
# 還原時，帳號home目錄存在則不還原，並記錄於 /mnt/big_account_list_fail2
# 主要是避免工程師手動還原，過一陣子因為此程式，又再一次還原

echo "此帳號 $line 家目錄存在，因此不進行還原，請手動確認原因。" >> /mnt/big_account_list_fail2
sed -i "/$line/d" /mnt/big_account_list2

elif [ ! -e /mnt/account2/$line/homedir/public_html ]; then
# 還原時，帳號備份檔public_html不存在，判定備份有問題不進行還原，並記錄於 /mnt/big_account_list_fail2

echo "此帳號 $line 備份檔public_html不存在，判定備份有問題，因此不進行還原，請手動確認原因。" >> /mnt/big_account_list_fail2
sed -i "/$line/d" /mnt/big_account_list2

else

# 將帳號 public_html, mail 先移到 /mnt/account2_tmp
mv -v /mnt/account2/"$line"/homedir/public_html /mnt/account2_tmp/
mv -v /mnt/account2/"$line"/homedir/mail /mnt/account2_tmp/

# 執行帳號還原
/scripts/restorepkg --force --allow_reseller /mnt/account2/"$line"

##########

if [ ! -e /home/$line ]; then
# 還原後帳號home不存在，表示還原失敗，則將 public_html,mail 移回原本目錄，並記錄於 /mnt/big_account_list_fail2，此帳號跳過接下來流程

mv -v /mnt/account2_tmp/public_html /mnt/account2/"$line"/homedir/
mv -v /mnt/account2_tmp/mail /mnt/account2/"$line"/homedir/
echo "還原後帳號home不存在，此帳號 $line 還原失敗，請手動確認原因" >> /mnt/big_account_list_fail2
sed -i "/$line/d" /mnt/big_account_list2

else

# 先刪除 /home/xxx/public_html, mail，再使用 mv 將備份 public_html, mail 搬移到 /home 帳號目錄底下
rm -rvf /home/"$line"/public_html
rm -rvf /home/"$line"/mail
mv -vf /mnt/account2_tmp/public_html /home/"$line"/
mv -vf /mnt/account2_tmp/mail /home/"$line"/

# 修正目錄與檔案權限
find /home/"$line"/public_html -type d -exec chmod 755 {} \;
find /home/"$line"/public_html -type f | xargs -d$'\n' -r chmod 644
find /home/"$line"/public_html -name '*.cgi' -o -name '*.pl' -o -name '*.perl' | xargs -r chmod 755
chown -R $line:$line /home/"$line"/public_html/
chown -R $line:$line /home/"$line"/mail/
find /home/"$line"/public_html -name .htaccess -exec chown $line.$line {} \;

chown $line:nobody /home/"$line"/public_html
chmod 750 /home/"$line"/public_html

# 修正沒有放在 public_html 範圍內網站根目錄權限
for SUBDOMAIN in $(grep -i documentroot /var/cpanel/userdata/"$line"/* | grep -v '.cache\|_SSL' | awk '{print $2}' | grep -v public_html)
do
  find $SUBDOMAIN -type d -exec chmod 755 {} \;
  find $SUBDOMAIN -type f | xargs -d$'\n' -r chmod 644
  find $SUBDOMAIN -name '*.cgi' -o -name '*.pl' -o -name '*.perl' | xargs -r chmod 755

  chown -R $line:$line $SUBDOMAIN
  find $SUBDOMAIN -name .htaccess -exec chown $line.$line {} \;
  chmod 750 $SUBDOMAIN
  chown $line:nobody $SUBDOMAIN
done

# 修正放在 public_html 範圍內網站根目錄權限
for SUBDOMAIN in $(grep -i documentroot /var/cpanel/userdata/"$line"/* | grep -v '.cache\|_SSL' | awk '{print $2}' | grep public_html)
do
  chmod 750 $SUBDOMAIN
  chown $line:nobody $SUBDOMAIN
done

# 如果轉移後IP不變，有新增 /tmp/auto_set_ip 檔案，就會自動套用備份檔案內原本設定之IP位址
if [ -e /tmp/auto_set_ip ]; then

cd /mnt/account2/"$line"/cp
IP=`cat "$line" | grep '^IP=' | awk -F\= '{print $2}'`
/usr/local/cpanel/bin/setsiteip -u "$line" "$IP"

fi

# 刪除該帳號備份檔，並更新還原進度清單
rm -rf /mnt/account2/"$line"
sed -i "/$line/d" /mnt/big_account_list2

fi

fi

# 加上檔案判斷，想要執行至一半中止，可新增 /tmp/stop2 檔案
# 如果有先手動還原某個帳號，請手動更新 big_account_list2 帳號列表，再一次手動執行此程式，即可繼續進行剩下的帳號還原作業
if [ -e /tmp/stop2 ]; then

echo ""
echo "目前還原至 $line 此帳號完成，程序已停止"
echo ""
rm -vf /tmp/stop2
exit

fi

done

date >> /mnt/restore_time2.txt

cp /mnt/restore2.log /mnt/restore2_`date +%Y%m%d%H%M`.log

