#!/bin/bash
# cpanel Fix Account Permissions
# https://github.com/PeachFlame/cPanel-fixperms
# https://www.thecpaneladmin.com/fix-account-permissions/
# 帳號備份檔，請放置 /mnt/account 目錄，最好是放在SSD硬碟
# 執行此 Script 之前，請先執行 find_sort_accounts2.sh 產生帳號列表

LOG_FILE="/mnt/restore2.log"
ACCOUNT_LIST="/mnt/big_account_list2"
ACCOUNT_LIST_FAIL="/mnt/big_account_list_fail2"
ACCOUNT_TMP="/mnt/account2_tmp"
COUNT=$(cat $ACCOUNT_LIST | wc -l)
COUNT_NOW="0"

mkdir -p /mnt/account2_tmp

rm -f "$LOG_FILE"

echo '' >> "$LOG_FILE"
echo '開始執行程式時間: ' `date` >> "$LOG_FILE"
echo '' >> "$LOG_FILE"
echo '====================' >> "$LOG_FILE"

exec < "$ACCOUNT_LIST"
while read line
do

  # 檢查 MySQL 連線狀態
  mysqladmin --defaults-file=/root/.my.cnf ping &>/dev/null

  if [ $? -eq 0 ]; then
	  
    echo '' >> "$LOG_FILE"
    echo "MySQL 服務目前正常." >> "$LOG_FILE"
  else
    echo ''
    echo "MySQL 服務目前異常 !!! 請先排除此問題，再重新執行 sh /opt/cpinst/restore_script/restore_public_html.sh 程式繼續還原."
    echo ''
    echo "請執行 systemctl status mysqld 或 journalctl -xeu mysqld.service 排查原因"
    echo ''
    echo -e "MySQL 服務目前異常 !!! 請先排除此問題，再重新執行 sh /opt/cpinst/restore_script/restore_public_html.sh 程式繼續還原.\n\n請執行 systemctl status mysqld 或 journalctl -xeu mysqld.service 排查原因" | mutt -s "主機轉移還原作業 restore2 發生異常" t5q1l6c4f4g4o1i3@twnoc.slack.com,x-aaaanppyuyhlhmabcfvytfwbjy@twnoc.slack.com
    exit 1

  fi

  if [ -e /home/"$line" ]; then

    # 還原時，帳號home目錄存在則不還原，並記錄於 /mnt/big_account_list_fail2
    # 主要是避免工程師手動還原，過一陣子因為此程式，又再一次還原
    echo "此帳號 $line 家目錄存在，因此不進行還原，請手動確認原因。" | tee -a "$ACCOUNT_LIST_FAIL"
    sed -i "/$line/d" "$ACCOUNT_LIST"

  elif [ ! -e /mnt/account2/"$line"/homedir/public_html ]; then

    # 還原時，帳號備份檔public_html不存在，判定備份有問題不進行還原，並記錄於 /mnt/big_account_list_fail2
    echo "此帳號 $line 備份檔public_html不存在，判定備份有問題，因此不進行還原，請手動確認原因。" | tee -a "$ACCOUNT_LIST_FAIL"
    sed -i "/$line/d" "$ACCOUNT_LIST"

  else

    echo '' >> "$LOG_FILE"
    echo '====================' >> "$LOG_FILE"
    echo '' >> "$LOG_FILE"
    echo "$line 帳號還原開始: " `date` >> "$LOG_FILE"
    echo '' >> "$LOG_FILE"

    # 將帳號 public_html, mail 先移到 /mnt/account2_tmp
    mv -v /mnt/account2/"$line"/homedir/public_html "$ACCOUNT_TMP" >> "$LOG_FILE"
    mv -v /mnt/account2/"$line"/homedir/mail "$ACCOUNT_TMP" >> "$LOG_FILE"

    # 執行帳號還原
    /scripts/restorepkg --force --allow_reseller /mnt/account2/"$line" >> "$LOG_FILE"

    if [ ! -e /home/"$line" ]; then

      # 還原後帳號home不存在，表示還原失敗，則將 public_html,mail 移回原本目錄，並記錄於 /mnt/big_account_list_fail2，此帳號跳過接下來流程
      mv "$ACCOUNT_TMP"/public_html /mnt/account2/"$line"/homedir/
      mv "$ACCOUNT_TMP"/mail /mnt/account2/"$line"/homedir/
      echo "還原後帳號home不存在，此帳號 $line 還原失敗，請手動確認原因" | tee -a "$ACCOUNT_LIST_FAIL"
      sed -i "/$line/d" "$ACCOUNT_LIST"

    else

      # 先刪除 /home/xxx/public_html, mail，再使用 mv 將備份 public_html, mail 搬移到 /home 帳號目錄底下
      rm -rvf /home/"$line"/public_html >> "$LOG_FILE"
      rm -rvf /home/"$line"/mail >> "$LOG_FILE"
      mv -vf "$ACCOUNT_TMP"/public_html /home/"$line"/ >> "$LOG_FILE"
      mv -vf "$ACCOUNT_TMP"/mail /home/"$line"/ >> "$LOG_FILE"

      # 修正目錄與檔案權限
      find /home/"$line"/public_html -type d -exec chmod 755 {} \;
      find /home/"$line"/public_html -type f | xargs -d$'\n' -r chmod 644
      find /home/"$line"/public_html -name '*.cgi' -o -name '*.pl' -o -name '*.perl' | xargs -r chmod 755
      chown -R $line:$line /home/"$line"/public_html/
      chown -R $line:$line /home/"$line"/mail/
      find /home/"$line"/public_html -name .htaccess -exec chown $line.$line {} \;

      chown $line:nobody /home/"$line"/public_html
      chmod 750 /home/"$line"/public_html

      # 修正沒有放在 public_html 範圍內網站根目錄權限
      for SUBDOMAIN in $(grep -i documentroot /var/cpanel/userdata/"$line"/* | grep -v '.cache\|_SSL' | awk '{print $2}' | grep -v public_html)
      do
        find $SUBDOMAIN -type d -exec chmod 755 {} \;
        find $SUBDOMAIN -type f | xargs -d$'\n' -r chmod 644
        find $SUBDOMAIN -name '*.cgi' -o -name '*.pl' -o -name '*.perl' | xargs -r chmod 755

        chown -R $line:$line $SUBDOMAIN
        find $SUBDOMAIN -name .htaccess -exec chown $line.$line {} \;
        chmod 750 $SUBDOMAIN
        chown $line:nobody $SUBDOMAIN
      done

      # 修正放在 public_html 範圍內網站根目錄權限
      for SUBDOMAIN in $(grep -i documentroot /var/cpanel/userdata/"$line"/* | grep -v '.cache\|_SSL' | awk '{print $2}' | grep public_html)
      do
        chmod 750 $SUBDOMAIN
        chown $line:nobody $SUBDOMAIN
      done

      # 如果轉移後IP不變，有新增 /tmp/auto_set_ip 檔案，就會自動套用備份檔案內原本設定之IP位址
      if [ -e /tmp/auto_set_ip ]; then

        cd /mnt/account2/"$line"/cp
        IP=`cat "$line" | grep '^IP=' | awk -F\= '{print $2}'`
        /usr/local/cpanel/bin/setsiteip -u "$line" "$IP" >> "$LOG_FILE"

      fi

      # 刪除該帳號備份檔，並更新還原進度清單
      rm -rvf /mnt/account2/"$line" >> "$LOG_FILE"
      sed -i "/$line/d" "$ACCOUNT_LIST"

      echo '' >> "$LOG_FILE"
      echo "$line 帳號還原結束: " `date` >> "$LOG_FILE"
      echo '' >> "$LOG_FILE"
      echo '====================' >> "$LOG_FILE"

      COUNT_NOW=$(($COUNT_NOW+1))
      COUNT_SUM=$(($COUNT-$COUNT_NOW))
      echo "$COUNT_NOW / $COUNT $line 帳號還原已完成 ( 還剩下 $COUNT_SUM 帳號尚未還原 )"
      
    fi

  fi

  if [ -e /tmp/stop2 ]; then

    # 加上檔案判斷，想要執行至一半中止，可新增 /tmp/stop 檔案
    # 如果有先手動還原某個帳號，請手動更新 big_account_list2 帳號列表，再一次手動執行此程式，即可繼續進行剩下的帳號還原作業
    echo ""
    echo "目前還原至 $line 此帳號完成，程序已停止"
    echo ""
    rm -vf /tmp/stop2
    exit

  fi

done

echo '' >> "$LOG_FILE"
echo '結束執行程式時間: ' `date` >> "$LOG_FILE"
echo '' >> "$LOG_FILE"

cp "$LOG_FILE" /mnt/restore_`date +%Y%m%d%H%M`.log
